Package: ParallelLogger
Type: Package
Title: Support for Parallel Computation, Logging, and Function
        Automation
Version: 3.5.1
Date: 2025-10-29
Authors@R: c(
  person("Martijn", "Schuemie", email = "schuemie@ohdsi.org", role = c("aut", "cre")),
  person("Marc", "Suchard", role = c("aut")),
  person("Observational Health Data Science and Informatics", role = c("cph"))
  )
Maintainer: Martijn Schuemie <schuemie@ohdsi.org>
Description: Support for parallel computation with progress bar, and option to stop or proceed on errors. Also provides logging to console and disk,
  and the logging persists in the parallel threads. Additional functions support function call automation with delayed execution (e.g. for executing functions in
  parallel).
License: Apache License 2.0
VignetteBuilder: knitr
Depends: R (>= 4.0.0)
Imports: snow, xml2, jsonlite, methods, utils, rstudioapi, memuse,
        parallel
Suggests: sendmailR, testthat, shiny, DT, knitr, rmarkdown, tibble
URL: https://ohdsi.github.io/ParallelLogger/,
        https://github.com/OHDSI/ParallelLogger
BugReports: https://github.com/OHDSI/ParallelLogger/issues
NeedsCompilation: no
RoxygenNote: 7.3.3
Encoding: UTF-8
Packaged: 2025-10-29 10:09:55 UTC; schuemie
Author: Martijn Schuemie [aut, cre],
  Marc Suchard [aut],
  Observational Health Data Science and Informatics [cph]
Repository: CRAN
Date/Publication: 2025-10-29 10:40:02 UTC
Built: R 4.6.0; ; 2026-04-25 23:21:29 UTC; unix
