# Generated by roxygen2: do not edit by hand

export(addDefaultConsoleLogger)
export(addDefaultEmailLogger)
export(addDefaultErrorReportLogger)
export(addDefaultFileLogger)
export(clearLoggers)
export(clusterApply)
export(clusterRequire)
export(convertJsonToSettings)
export(convertSettingsToJson)
export(createArgFunction)
export(createConsoleAppender)
export(createEmailAppender)
export(createFileAppender)
export(createLogger)
export(excludeFromList)
export(getLoggers)
export(getPhysicalMemory)
export(getThreadNumber)
export(launchLogViewer)
export(layoutEmail)
export(layoutErrorReport)
export(layoutParallel)
export(layoutSimple)
export(layoutStackTrace)
export(layoutTimestamp)
export(loadSettingsFromJson)
export(logDebug)
export(logError)
export(logFatal)
export(logInfo)
export(logTrace)
export(logWarn)
export(makeCluster)
export(matchInList)
export(registerLogger)
export(saveSettingsToJson)
export(selectFromList)
export(stopCluster)
export(unregisterLogger)
importFrom(methods,getFunction)
importFrom(methods,is)
importFrom(rstudioapi,getActiveDocumentContext)
importFrom(utils,capture.output)
importFrom(utils,help)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,limitedLabels)
importFrom(utils,memory.limit)
importFrom(utils,menu)
importFrom(utils,sessionInfo)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
